/**
  * =========================================================================
  *  Copyright (C) since 1995 by Leidson CAF & PlanetaMessenger.org
  *
  *  This module is developed and maintained by PlanetaMessenger.org. 
  *  Specs, New and updated versions can be found in 
  *  http://www.planetamessenger.org
  *  If you want contact the Team please send a email to Project Manager
  *  Leidson Campos Alves Ferreira at leidson@planetamessenger.org
  *
  *  Copyright (C) since 1995 by PlanetaMessenger.org
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  * ===========================================================================
  */

/**
  *
  * $Id: custommmlfile.cpp 49 2011-03-09 00:14:32Z popolony2k $
  * $Author: popolony2k $
  * $Date: 2011-03-09 00:14:32 +0000 (Wed, 09 Mar 2011) $
  * $Revision: 49 $
  * $HeadURL: file:///svn/p/oldskooltech/code/msx/trunk/tools/cpp/sound/mml2msd/custommmlfile.cpp $
  */

#include "custommmlfile.h"
#include "threemltokens.h"
#include "msdtokens.h"
#include "stringutil.h"


/**
  * Constructor. Initializes all class data.
  * @param pCnv Pointer to object used to perform MML conversion;
  */
CustomMMLFile :: CustomMMLFile( IBase *pCnv )  {

  CustomMMLFile :: pCnv = pCnv;
}

/**
  * Destructor. Finalize all class data.
  */
CustomMMLFile :: ~CustomMMLFile( void )  {

  Close();  
}

/**
  * Open a custom MML file.
  * @param strCustomMMLFileName The custom MML file to open;
  */
bool CustomMMLFile :: Open( string strCustomMMLFileName )  {

  // Open input custom MML file
  in.open( strCustomMMLFileName.c_str(), ios_base :: in );

  if( !in.is_open() )
    return false;

  return true; 
}

/**
  * Close the custom MML file opened by @see Open();
  */
void CustomMMLFile :: Close( void )  {

  if( in.is_open() )
    in.close();
}

/**
  * Execute the custom MML parsing, adding the custom MML stored to
  * channels on conversion object;
  */
void CustomMMLFile :: Run( void )  {
  
  string       strToken;
  size_t       nCharPos;

  while( in.good() )  {
    getline( in, strToken );

    ::charcase( strToken, UPPERCASE );
    
    nCharPos = strToken.find( THREEML_TOKEN_ASSIGN_VALUE );
    
    if( nCharPos != string :: npos )  {

      string     strKey;
      string     strValue( "" );
      int        nChannelId = -1;

      /*
       * Get Key and Value.
       */
      strKey = strToken.substr( 0, nCharPos );
      nCharPos++;

      if( nCharPos < strToken.size() )
        strValue = strToken.substr( nCharPos );
        
      ::trim( strKey );
      
      /*
       * Check the key to retrieve.
       */
      if( strKey.compare( MSD_TOKEN_FM1 ) == 0 )
        nChannelId = 1;
      else
      if( strKey.compare( MSD_TOKEN_FM2 ) == 0 )
        nChannelId = 2;
      else
      if( strKey.compare( MSD_TOKEN_FM3 ) == 0 )
        nChannelId = 3;
      else
      if( strKey.compare( MSD_TOKEN_FM4 ) == 0 )
        nChannelId = 4;
      else
      if( strKey.compare( MSD_TOKEN_FM5 ) == 0 )
        nChannelId = 5;
      else
      if( strKey.compare( MSD_TOKEN_FM6 ) == 0 )
        nChannelId = 6;
      else
      if( strKey.compare( MSD_TOKEN_FMR ) == 0 )
        nChannelId = -1;
      else
      if( strKey.compare( MSD_TOKEN_FM7 ) == 0 )
        nChannelId = 7;
      else
      if( strKey.compare( MSD_TOKEN_FM8 ) == 0 )
        nChannelId = 8;
      else
      if( strKey.compare( MSD_TOKEN_FM9 ) == 0 )
        nChannelId = 9;
      else
      if( strKey.compare( MSD_TOKEN_PSG1 ) == 0 )
        nChannelId = 10;
      else
      if( strKey.compare( MSD_TOKEN_PSG2 ) == 0 )
        nChannelId = 11;
      else
      if( strKey.compare( MSD_TOKEN_PSG3 ) == 0 )
        nChannelId = 12;
      else
      if( strKey.compare( MSD_TOKEN_SCC1 ) == 0 )
        nChannelId = 13;
      else
      if( strKey.compare( MSD_TOKEN_SCC2 ) == 0 )
        nChannelId = 14;
      else
      if( strKey.compare( MSD_TOKEN_SCC3 ) == 0 )
        nChannelId = 15;
      else
      if( strKey.compare( MSD_TOKEN_SCC4 ) == 0 )
        nChannelId = 16;
      else
      if( strKey.compare( MSD_TOKEN_SCC5 ) == 0 )
        nChannelId = 17;

      if( nChannelId != -1 )
        pCnv -> AddCustomMML( strValue, nChannelId );
    }
  }
}
