/**
  * =========================================================================
  *  Copyright (C) since 1995 by Leidson CAF & PlanetaMessenger.org
  *
  *  This module is developed and maintained by PlanetaMessenger.org. 
  *  Specs, New and updated versions can be found in 
  *  http://www.planetamessenger.org
  *  If you want contact the Team please send a email to Project Manager
  *  Leidson Campos Alves Ferreira at leidson@planetamessenger.org
  *
  *  Copyright (C) since 1995 by PlanetaMessenger.org
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  * ===========================================================================
  */

#include <cstdlib>
#include "threeml.h"
#include "tui.h"
#include "custommmlfile.h"


/**
  * Program entry point.
  * @param argc The argument counter;
  * @param argv The parameter array of parameters;
  */
int main( int argc, char *argv[] ) {
  
  ArgV       args;
  TUI        *pUI;
  
  if( argc > 1 )  {

    for( int nCount = 1; nCount < argc; nCount++ )
      args.push_back( argv[nCount] );

    pUI = new TUI( args );

    if( !pUI -> HasInvalidArguments() && !pUI -> GetHelp() )  {

      TUI :: ShowHeader();

      /*
       * The IBase object is used here because in future we can use this 
       * conversion tool with new Conversion objects.
       */
      IBase     *pConv = new ThreeML( pUI -> GetInputFileName(), pUI -> GetOutputFileName() );

      /*
       * Custom MML processing.
       */
      if( pUI -> HasCustomMML() )
        pConv -> AddCustomMML( pUI -> GetCustomMML() );

      /*
       * Start of custom MML file processing.
       */
      if( pUI -> HasCustomMMLFileName() )  {
        CustomMMLFile     customMMLFile( pConv );
        
        if( customMMLFile.Open( pUI -> GetCustomMMLFileName() ) )  {
          customMMLFile.Run();
          customMMLFile.Close();
        }
        else
          cout << "main() - Error to open custom MML input file [" << pUI -> GetCustomMMLFileName().c_str() << "]";
      }

      /*
       * Find Tempo, remarks and verbose processing.
       */
      pConv -> SetFindTempo( pUI -> GetFindTempo() );
      pConv -> SetVerbose( pUI -> GetVerbose() );
      pConv -> SetRemarks( pUI -> GetRemarks() );
      pConv -> SetRemoveInstruments( pUI -> GetRemoveInstruments() );

      if( pConv -> Open() )  {
        pConv -> Run();
        pConv -> Close();
      }

      delete pConv;
    }
    else  {
      TUI :: ShowHelp();

      if( pUI -> HasInvalidArguments() )
        cout << pUI -> GetLastError();
    }

    delete pUI;
  }
  else
    TUI :: ShowHelp();

  return EXIT_SUCCESS;
}
